<?php
/*+**********************************************************************************
 * The contents of this file are subject to the vtiger CRM Public License Version 5.4.0
 * ("License"); You may not use this file except in compliance with the License
 ************************************************************************************/
include_once dirname(__FILE__) . '/../ISMSProvider.php';
include_once 'vtlib/Vtiger/Net/Client.php';

class KwnSms implements ISMSProvider {
	
	private $_username;
	private $_password;
	private $_parameters = array();
	
	const SERVICE_URI = 'http://api.kenweini.com';
	private static $REQUIRED_PARAMETERS = array('api_id', 'from', 'mo');
	
	function __construct() {		
	}
	
	public function setAuthParameters($username, $password) {
		$this->_username = $username;
		$this->_password = $password;
	}
	
	public function setParameter($key, $value) {
		$this->_parameters[$key] = $value;
	}
	
	public function getParameter($key, $defvalue = false)  {
		if(isset($this->_parameters[$key])) {
			return $this->_parameters[$key];
		}
		return $defvalue;
	}
	
	public function getRequiredParams() {
		return self::$REQUIRED_PARAMETERS;
	}
	
	public function getServiceURL($type = false) {		
		if($type) {
			switch(strtoupper($type)) {				
				case self::SERVICE_AUTH: return  self::SERVICE_URI . '/http/auth';
				case self::SERVICE_SEND: return  self::SERVICE_URI . '/http/sendmsg';
				case self::SERVICE_QUERY: return self::SERVICE_URI . '/http/querymsg';			
			}
		}
		return false;
	}
	
	protected function prepareParameters() {
		$params = array('user' => $this->_username, 'password' => $this->_password);
		foreach (self::$REQUIRED_PARAMETERS as $key) {
			$params[$key] = $this->getParameter($key);
		}
		return $params;
	}
	
	public function send($message, $tonumbers) {
		if(!is_array($tonumbers)) {
			$tonumbers = array($tonumbers);
		}
		
		$params = $this->prepareParameters();
		$params['text'] = $message;
		$params['to'] = implode(',', $tonumbers);
		
		define('DRUPAL_ROOT', $_SERVER['DOCUMENT_ROOT']);
		include_once DRUPAL_ROOT.'/includes/bootstrap.inc';
		drupal_bootstrap(DRUPAL_BOOTSTRAP_FULL);
		$account = user_load_by_name($params['user']);		
		$query = db_select('sms_user', 'su')
			->fields('su', array('smsuser','smspassword','smschannel','tiaoshu','vip'))
			->condition('uid', $account->uid);
		$tablesmsuser = $query->execute()->fetchAssoc(); 
		if (!empty($tablesmsuser['smschannel'])){
			$channelv = variable_get($tablesmsuser['smschannel']);
			$cmschannelarr = unserialize($channelv['channelarr']);
		}else{
			return $results[0]['error'] = true; 
		}
		if ($tablesmsuser['tiaoshu'] <= 0){
			return $results[0]['error'] = true; 
		}
		if ($cmschannelarr['smsencode'] != 'UTF-8') $Msg = @iconv('UTF-8', $cmschannelarr['smsencode'], $params['text']);
		$sendmessage = sms_cc_sms_send($tablesmsuser['smschannel'],$tablesmsuser['smsuser'],$tablesmsuser['smspassword'],$params['to'],$Msg, $cmschannelarr['smsaction']['smssend']);	
		if ($cmschannelarr['smsencode'] != 'UTF-8') $sendmessage = @iconv($cmschannelarr['smsencode'], 'UTF-8', $sendmessage);
		
		$match_name = $cmschannelarr['okpreg']; 
		preg_match($match_name, $sendmessage ,$m);
		$result = array( 'error' => false, 'statusmessage' => '' );
		$results = array();
		if (!empty($m[1]) && $m[1] == $cmschannelarr['okpregresult']){
			$permsg = $cmschannelarr['textnum'];
			$msglen = drupal_strlen($params['text']);
			$msgshu = ceil($msglen/$permsg); //͵Ϊ;
			$sendnum = count($tonumbers);  //һη͵Ⱥ
			$tnumber = $sendnum*$msgshu; //͵ܶ
			$tiaoshu = $tablesmsuser['tiaoshu']-$tnumber;	
			if ($tiaoshu <= 0){
				$tiaoshu = 0; 
			}
			db_update('sms_user')
				->fields(array('tiaoshu' => $tiaoshu))
				->condition('uid', $account->uid)
				->execute();
			
			$result['id'] = trim($m[1]);
			$result['to'] = $params['to'];
			$result['statusmessage'] = 'OK';
			$result['status'] = self::MSG_STATUS_PROCESSING;
		}else{
			$match_namee = $cmschannelarr['errpreg']; 
			preg_match($match_namee, $sendmessage ,$me);
			$errmessage = !empty($me[1])?$me[1]:t('Unknown ');
			$result['error'] = true; 
			$result['to'] = $params['to'];
			$result['statusmessage'] = $errmessage; // Complete error message
		}
		
		$results[0] = $result;
		return $results;
	}
	
	public function query($messageid) {

		$params = $this->prepareParameters();
		$params['apimsgid'] = $messageid;

		$serviceURL = $this->getServiceURL(self::SERVICE_QUERY);
		$httpClient = new Vtiger_Net_Client($serviceURL);					
		$response = $httpClient->doPost($params);
		
		$response = trim($response);

		$result = array( 'error' => false, 'needlookup' => 1, 'statusmessage' => '' );
		
		if(preg_match("/ERR: (.*)/", $response, $matches)) {
			$result['error'] = true;
			$result['needlookup'] = 0;
			$result['statusmessage'] = $matches[0];
			
		} else if(preg_match("/ID: ([^ ]+) Status: ([^ ]+)/", $response, $matches)) {
			$result['id'] = trim($matches[1]);
			$status = trim($matches[2]);

			// Capture the status code as message by default.
			$result['statusmessage'] = "CODE: $status";

			if($status == '002' || $status == '008' || $status == '011' ) {
				$result['status'] = self::MSG_STATUS_PROCESSING;
			} else if($status == '003' || $status == '004') {
				$result['status'] = self::MSG_STATUS_DISPATCHED;
				$result['needlookup'] = 0;
			} else {
				$statusMessage = "";
				switch($status) {
				case '001': $statusMessage = 'Message unknown';                 $needlookup = 0; break;
				case '005': $statusMessage = 'Error with message';              $needlookup = 0; break;
				case '006': $statusMessage = 'User cancelled message delivery'; $needlookup = 0; break;
				case '007': $statusMessage = 'Error delivering message';        $needlookup = 0; break;
				case '009': $statusMessage = 'Routing error';                   $needlookup = 0; break;
				case '010': $statusMessage = 'Message expired';                 $needlookup = 0; break;
				case '012': $statusMessage = 'Out of credit';                   $needlookup = 0; break;
				}			
				if(!empty($statusMessage)) {		
					$result['error'] = true;
					$result['needlookup'] = $needlookup;
					$result['statusmessage'] = $statusmessage;
				}
			}
		} 	
		return $result;
	}
}
?>
